import React from 'react';

function _defineProperty(obj, key, value) {
  if (key in obj) {
    Object.defineProperty(obj, key, {
      value: value,
      enumerable: true,
      configurable: true,
      writable: true
    });
  } else {
    obj[key] = value;
  }

  return obj;
}

function ownKeys(object, enumerableOnly) {
  var keys = Object.keys(object);

  if (Object.getOwnPropertySymbols) {
    var symbols = Object.getOwnPropertySymbols(object);
    if (enumerableOnly) symbols = symbols.filter(function (sym) {
      return Object.getOwnPropertyDescriptor(object, sym).enumerable;
    });
    keys.push.apply(keys, symbols);
  }

  return keys;
}

function _objectSpread2(target) {
  for (var i = 1; i < arguments.length; i++) {
    var source = arguments[i] != null ? arguments[i] : {};

    if (i % 2) {
      ownKeys(Object(source), true).forEach(function (key) {
        _defineProperty(target, key, source[key]);
      });
    } else if (Object.getOwnPropertyDescriptors) {
      Object.defineProperties(target, Object.getOwnPropertyDescriptors(source));
    } else {
      ownKeys(Object(source)).forEach(function (key) {
        Object.defineProperty(target, key, Object.getOwnPropertyDescriptor(source, key));
      });
    }
  }

  return target;
}

var rootContext = null;

var createRootContext = () => {
  return React.createContext({
    addInnerContext: contextId => {},
    getInnerContext: contextId => {},
    addMultipleInnerContexts: contextIds => {},
    removeMultipleInnerContexts: contextIds => {},
    removeInnerContext: contextId => {},
    setInnerState: (contextId, state) => {},
    getInnerState: contextId => {}
  });
};

var getRootContext = () => {
  if (rootContext) {
    return rootContext;
  }

  console.log("creating root context");
  rootContext = createRootContext();
  return rootContext;
};

class MultiContext extends React.Component {
  constructor(props) {
    super(props);
    this.addInnerContext = this.addInnerContext.bind(this);
    this.getInnerContext = this.getInnerContext.bind(this);
    this.setInnerState = this.setInnerState.bind(this);
    this.getInnerState = this.getInnerState.bind(this);
    this.removeInnerContext = this.removeInnerContext.bind(this);
    this.addMultipleInnerContexts = this.addMultipleInnerContexts.bind(this);
    this.removeMultipleInnerContexts = this.removeMultipleInnerContexts.bind(this);
    this.state = {
      innerContexts: {},
      root: {
        addInnerContext: this.addInnerContext,
        getInnerContext: this.getInnerContext,
        removeInnerContext: this.removeInnerContext,
        setInnerState: this.setInnerState,
        getInnerState: this.getInnerState,
        addMultipleInnerContexts: this.addMultipleInnerContexts,
        removeMultipleInnerContexts: this.removeMultipleInnerContexts
      }
    };
  }

  removeMultipleInnerContexts(contextIds) {
    this.setState((state, props) => {
      var st = _objectSpread2({}, state.innerContexts);

      for (var k of contextIds) {
        delete st[k];
      }

      return {
        innerContexts: st
      };
    });
  }

  addMultipleInnerContexts(contextIds, optContexts) {
    //avoid redraws by doing adding and removing more contexts at once
    var contexts = {};
    this.setState((state, props) => {
      var st = _objectSpread2({}, state.innerContexts);

      for (var j = 0; j < contextIds.length; j++) {
        var k = contextIds[j];
        var ex = state.innerContexts[k] && state.innerContexts[k].context;
        if (ex) continue;
        var ctx = optContexts && optContexts[j] ? optContexts[j] : React.createContext(null);
        st[k] = {
          context: ctx,
          state: null
        };
        contexts[k] = ctx;
      }

      return {
        innerContexts: st
      };
    });
    return contexts;
  }

  addInnerContext(contextId, optContext) {
    var ctx = optContext ? optContext : React.createContext(null);
    this.setState((state, props) => {
      var ex = state.innerContexts[contextId] && state.innerContexts[contextId].context;
      if (ex) return {};

      var st = _objectSpread2({}, state.innerContexts);

      st[contextId] = {
        context: ctx,
        state: null
      };
      return {
        innerContexts: st
      };
    });
    return ctx;
  }

  removeInnerContext(contextId) {
    this.setState((state, props) => {
      if (!state.innerContexts[contextId]) return null;

      var st = _objectSpread2({}, state.innerContexts);

      delete st[contextId];
      return {
        innerContexts: st
      };
    });
  }

  getInnerContext(contextId) {
    return this.state.innerContexts[contextId] && this.state.innerContexts[contextId].context;
  }

  addProvider(innerProvider, contextId) {
    var PRV = this.getInnerContext(contextId).Provider;
    var innerState = this.state.innerContexts[contextId].state;
    return /*#__PURE__*/React.createElement(PRV, {
      value: innerState
    }, innerProvider);
  }

  setInnerState(contextId, stateF) {
    this.setState((state, props) => {
      var st = _objectSpread2({}, state.innerContexts);

      if (!st || !st[contextId]) {
        return {
          innerContexts: {}
        };
      }

      st[contextId].state = stateF(st[contextId].state);
      return {
        innerContexts: st
      };
    });
  }

  getInnerState(contextId) {
    return this.state.innerContexts[contextId] && this.state.innerContexts[contextId].state;
  }

  render() {
    var currProvider = this.props.children;

    for (var cid in this.state.innerContexts) {
      currProvider = this.addProvider(currProvider, cid);
    }
    /*
    Important! 
    If you use this library to create another library that will share the context with the application, you need to provide the rootContext as a param.
    The reason is that it is not possible to share the variables between the modules in WebPack, so the RootContext variable provider will not give any value in your lib
    */


    var Provider = this.props.rootContext ? this.props.rootContext.Provider : getRootContext().Provider;
    return /*#__PURE__*/React.createElement(Provider, {
      value: this.state.root
    }, currProvider);
  }

  get rootContext() {
    return getRootContext();
  }

  static get rootContext() {
    return getRootContext();
  }

}

/*
every new child context forces a redraw, sometimes messing with the components. To avoid this, we will use the MultiContext with the pre-populated contexts, and never remove the created ones, just free them. We will use the internal mapping for the contexts. Real contexts will be created with numbers 1,2,3 .... serving as a counter, and the oids will be stored in a map
*/

class ContextPool extends React.Component {
  constructor(props) {
    super(props);
    this.occupied = {};
    this.free = [];
    this.idCounter = 0;
    this.innerContexts = [];
    this.pending = {}; //adding inner contexts may happen before the actual inner context is ready, here we store the pending contexts

    this.addInnerContext = this.addInnerContext.bind(this);
    this.getInnerContext = this.getInnerContext.bind(this);
    this.setInnerState = this.setInnerState.bind(this);
    this.getInnerState = this.getInnerState.bind(this);
    this.removeInnerContext = this.removeInnerContext.bind(this);
    this.addMultipleInnerContexts = this.addMultipleInnerContexts.bind(this);
    this.removeMultipleInnerContexts = this.removeMultipleInnerContexts.bind(this);
  }

  render() {
    var Provider = this.props.rootContext.Provider;
    return /*#__PURE__*/React.createElement(Provider, {
      value: this
    }, /*#__PURE__*/React.createElement(MultiContext, {
      ref: ctx => this.ctx = ctx
    }, this.props.children));
  }

  componentDidMount() {
    //add initial number of contexts as defined in the properties
    var pendingKeys = Object.keys(this.pending);
    var ids = [...Array(this.props.initialSize + pendingKeys.length).keys()];

    for (var j = 0; j < pendingKeys.length; j++) {
      this.innerContexts[j] = this.pending[pendingKeys[j]];
      this.occupied[pendingKeys[j]] = j;
    }

    this.idCounter = this.props.initialSize + pendingKeys.length;
    this.free = [...ids];
    this.free.splice(0, pendingKeys.length);

    for (var id of this.free) {
      this.innerContexts[id] = React.createContext(null);
    }

    this.pending = {};
    this.ctx.addMultipleInnerContexts(ids, this.innerContexts);
  }

  checkFree() {
    if (this.free.length < this.props.minimumFree) {
      var newIdCounter = this.idCounter + this.props.minimumFree;
      var newIds = [...Array(newIdCounter).keys()];
      newIds.splice(0, this.props.idCounter);
      var newContexts = [];

      for (var j of newIds) {
        var ctx = React.createContext(null);
        newContexts.push(ctx);
        this.innerContexts[j] = ctx;
      }

      this.idCounter = newIdCounter;
      this.free = this.free.concat(newIds);
      this.ctx.addMultipleInnerContexts(newIds, newContexts);
    }
  }

  addInnerContext(contextId, optContext) {
    if (!this.ctx) {
      var retCtx = optContext ? optContext : React.createContext(null);
      this.pending[contextId] = retCtx;
      return retCtx;
    }

    this.checkFree();
    var innerId = this.free.splice(0, 1)[0]; //remove first free

    this.occupied[contextId] = innerId;

    if (optContext) {
      //the free context is not used, replace it with the outside created one
      this.innerContexts[innerId] = optContext;
    }

    return this.innerContexts[innerId];
  }

  getInnerContext(contextId) {
    return this.innerContexts[this.occupied[contextId]];
  }

  removeInnerContext(contextId) {
    this.setInnerState(contextId, _ => null);
    var innerId = this.occupied[contextId];
    delete this.occupied[contextId];
    this.free.push(innerId);
  }

  setInnerState(contextId, stateF) {
    this.ctx.setInnerState(this.occupied[contextId], stateF);
  }

  getInnerState(contextId) {
    return this.ctx.getInnerState(this.occupied[contextId]);
  }

  addMultipleInnerContexts(contextIds, optContexts) {
    var rez = [];

    for (var i = 0; i < contextIds.length; i++) {
      var j = contextIds[i]; //looping is enough, there will be no redraws

      if (optContexts && optContexts[i]) {
        rez.push(this.addInnerContext(j, optContexts[i]));
      } else {
        rez.push(this.addInnerContext(j));
      }
    }

    return rez;
  }

  removeMultipleInnerContexts(contextIds) {
    for (var j of contextIds) {
      this.removeInnerContext(j);
    }
  }

  get rootContext() {
    return this.props.rootContext;
  }

}

export default MultiContext;
export { ContextPool };
//# sourceMappingURL=bundle.js.map
